From: Paul Mackerras <paulus@samba.org>
To: akpm@osdl.org
Cc: ntl@pobox.com, anton@samba.org, linux-kernel@vger.kernel.org
Subject: [PATCH] PPC64 rtasd shouldn't hold cpucontrol while sleeping
Date: Mon, 14 Mar 2005 14:23:39 +1100 [thread overview]
Message-ID: <16949.955.2292.852845@cargo.ozlabs.ibm.com> (raw)
This patch is from Nathan Lynch <ntl@pobox.com>.
The rtasd thread should not hold the cpucontrol semaphore while
sleeping between event scans in its first pass; it needlessly delays
boot by one second per cpu when CONFIG_HOTPLUG_CPU=y.
Signed-off-by: Nathan Lynch <ntl@pobox.com>
Signed-off-by: Paul Mackerras <paulus@samba.org>
--- linux-2.6.11.orig/arch/ppc64/kernel/rtasd.c 2005-02-20 02:08:49.000000000 +0000
+++ linux-2.6.11/arch/ppc64/kernel/rtasd.c 2005-02-20 18:13:54.000000000 +0000
@@ -400,10 +400,33 @@
} while(error == 0);
}
+static void do_event_scan_all_cpus(long delay)
+{
+ int cpu;
+
+ lock_cpu_hotplug();
+ cpu = first_cpu(cpu_online_map);
+ for (;;) {
+ set_cpus_allowed(current, cpumask_of_cpu(cpu));
+ do_event_scan(rtas_token("event-scan"));
+ set_cpus_allowed(current, CPU_MASK_ALL);
+
+ /* Drop hotplug lock, and sleep for the specified delay */
+ unlock_cpu_hotplug();
+ set_current_state(TASK_INTERRUPTIBLE);
+ schedule_timeout(delay);
+ lock_cpu_hotplug();
+
+ cpu = next_cpu(cpu, cpu_online_map);
+ if (cpu == NR_CPUS)
+ break;
+ }
+ unlock_cpu_hotplug();
+}
+
static int rtasd(void *unused)
{
unsigned int err_type;
- int cpu = 0;
int event_scan = rtas_token("event-scan");
int rc;
@@ -437,17 +460,7 @@
}
/* First pass. */
- lock_cpu_hotplug();
- for_each_online_cpu(cpu) {
- DEBUG("scheduling on %d\n", cpu);
- set_cpus_allowed(current, cpumask_of_cpu(cpu));
- DEBUG("watchdog scheduled on cpu %d\n", smp_processor_id());
-
- do_event_scan(event_scan);
- set_current_state(TASK_INTERRUPTIBLE);
- schedule_timeout(HZ);
- }
- unlock_cpu_hotplug();
+ do_event_scan_all_cpus(HZ);
if (surveillance_timeout != -1) {
DEBUG("enabling surveillance\n");
@@ -455,25 +468,11 @@
DEBUG("surveillance enabled\n");
}
- lock_cpu_hotplug();
- cpu = first_cpu(cpu_online_map);
- for (;;) {
- set_cpus_allowed(current, cpumask_of_cpu(cpu));
- do_event_scan(event_scan);
- set_cpus_allowed(current, CPU_MASK_ALL);
-
- /* Drop hotplug lock, and sleep for a bit (at least
- * one second since some machines have problems if we
- * call event-scan too quickly). */
- unlock_cpu_hotplug();
- set_current_state(TASK_INTERRUPTIBLE);
- schedule_timeout((HZ*60/rtas_event_scan_rate) / 2);
- lock_cpu_hotplug();
-
- cpu = next_cpu(cpu, cpu_online_map);
- if (cpu == NR_CPUS)
- cpu = first_cpu(cpu_online_map);
- }
+ /* Delay should be at least one second since some
+ * machines have problems if we call event-scan too
+ * quickly. */
+ for (;;)
+ do_event_scan_all_cpus((HZ*60/rtas_event_scan_rate) / 2);
error:
/* Should delete proc entries */
reply other threads:[~2005-03-14 3:23 UTC|newest]
Thread overview: [no followups] expand[flat|nested] mbox.gz Atom feed
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=16949.955.2292.852845@cargo.ozlabs.ibm.com \
--to=paulus@samba.org \
--cc=akpm@osdl.org \
--cc=anton@samba.org \
--cc=linux-kernel@vger.kernel.org \
--cc=ntl@pobox.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®